import os
import sys

sys.path.append(os.path.abspath('..'))
os.environ['DJANGO_SETTINGS_MODULE'] = 'fake_settings'

import jingo

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

extensions = ['sphinx.ext.autodoc']

# General information about the project.
project = u'Jingo'
copyright = u'2010-2015, The Mozilla Foundation'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# version: The short X.Y version.
# release: The full version, including alpha/beta/rc tags.
version = release = jingo.__version__

# List of directories, relative to source directory, that shouldn't be searched
# for source files.
exclude_trees = ['_build']



"""Adapter for using Jinja2 with Django."""

from __future__ import unicode_literals

import functools
import logging
import re

from django.apps import apps
from django.conf import settings
from django.template.base import Origin, TemplateDoesNotExist
from django.template.loader import BaseLoader
from django.utils.importlib import import_module

try:
    import importlib.util
    if hasattr(importlib.util, 'find_spec'):  # Py3>=3.4
        def has_helpers(path):
            return importlib.util.find_spec('helpers', path) is not None
    else:  # Py3<3.4
        def has_helpers(path):
            # For Python 3.3, just try to import the module. Unfortunately,
            # this changes the contract slightly for Python 3.3: if there is an
            # module but this raises a legitimate ImportError, jingo will act
            # as if the module doesn't exist. The intent is that we raise
            # legitimate ImportErrors but ignore missing modules.
            try:
                import_module('helpers', path)
                return True
            except ImportError:
                return False
except ImportError:
    import imp

    def has_helpers(path):
        try:
            imp.find_module('helpers', path)
            return True
        except ImportError:
            return False

import jinja2

try:
    from django.template.engine import Engine

    has_engine = True

    def get_standard_processors():
        return Engine.get_default().template_context_processors

except ImportError:
    from django.template.context import get_standard_processors
    has_engine = False

VERSION = (0, 8, 0)
__version__ = '.'.join(map(str, VERSION))

EXCLUDE_APPS = (
    'admin',
    'admindocs',
    'registration',
    'context_processors',
)

log = logging.getLogger('jingo')

_helpers_loaded = False


class Template(jinja2.Template):

    def render(self, context={}):
        """Render's a template, context can be a Django Context or a
        dictionary.
        """
        # flatten the Django Context into a single dictionary.
        context_dict = {}
        if hasattr(context, 'dicts'):
            for d in context.dicts:
                context_dict.update(d)
        else:
            context_dict = context

            # Django Debug Toolbar needs a RequestContext-like object in order
            # to inspect context.
            class FakeRequestContext:
                dicts = [context]
            context = FakeRequestContext()

        # Used by debug_toolbar.
        if settings.TEMPLATE_DEBUG:
            from django.test import signals
            self.origin = Origin(self.filename)
            signals.template_rendered.send(sender=self, template=self,
                                           context=context)

        return super(Template, self).render(context_dict)


class Environment(jinja2.Environment):
    template_class = Template

    def get_template(self, name, parent=None, globals=None):
        """Make sure our helpers get loaded before any templates."""
        load_helpers()
        return super(Environment, self).get_template(name, parent, globals)

    def from_string(self, source, globals=None, template_class=None):
        load_helpers()
        return super(Environment, self).from_string(source, globals,
                                                    template_class)


_env = None


def get_env():
    """Configure and return a jinja2 Environment."""
    global _env
    if _env:
        return _env
    # Mimic Django's setup by loading templates from directories in
    # TEMPLATE_DIRS and packages in INSTALLED_APPS.
    loaders = [jinja2.FileSystemLoader(d) for d in settings.TEMPLATE_DIRS]
    loaders += [jinja2.PackageLoader(c.name) for c in apps.get_app_configs()]

    opts = {
        'trim_blocks': True,
        'extensions': ['jinja2.ext.i18n'],
        'autoescape': True,
        'auto_reload': settings.DEBUG,
        'loader': jinja2.ChoiceLoader(loaders),
    }

    if hasattr(settings, 'JINJA_CONFIG'):
        if hasattr(settings.JINJA_CONFIG, '__call__'):
            config = settings.JINJA_CONFIG()
        else:
            config = settings.JINJA_CONFIG
        opts.update(config)

    e = Environment(**opts)
    # Install null translations since gettext isn't always loaded up during
    # testing.
    if ('jinja2.ext.i18n' in e.extensions or
            'jinja2.ext.InternationalizationExtension' in e.extensions):
        e.install_null_translations()
    _env = e
    return e


def render_to_string(request, template, context=None):
    """
    Render a template into a string.
    """
    def get_context():
        c = {} if context is None else context.copy()
        for processor in get_standard_processors():
            c.update(processor(request))
        return c

    # If it's not a Template, it must be a path to be loaded.
    if not isinstance(template, jinja2.environment.Template):
        template = get_env().get_template(template)

    return template.render(get_context())


def load_helpers():
    """Try to import ``helpers.py`` from each app in INSTALLED_APPS."""
    # We want to wait as long as possible to load helpers so there aren't any
    # weird circular imports with jingo.
    global _helpers_loaded
    if _helpers_loaded:
        return
    _helpers_loaded = True

    from jingo import helpers  # noqa

    for config in apps.get_app_configs():
        if not has_helpers(config.name):
            continue

        import_module('%s.helpers' % config.name)


class Register(object):
    """Decorators to add filters and functions to the template Environment."""
    def filter(self, f=None, override=True):
        """Adds the decorated function to Jinja's filter library."""
        def decorator(f):
            @functools.wraps(f)
            def wrapper(*args, **kw):
                return f(*args, **kw)
            return self.filter(wrapper, override)

        if not f:
            return decorator
        if override or f.__name__ not in get_env().filters:
            get_env().filters[f.__name__] = f
        return f

    def function(self, f=None, override=True):
        """Adds the decorated function to Jinja's global namespace."""
        def decorator(f):
            @functools.wraps(f)
            def wrapper(*args, **kw):
                return f(*args, **kw)
            return self.function(wrapper, override)

        if not f:
            return decorator
        if override or f.__name__ not in get_env().globals:
            get_env().globals[f.__name__] = f
        return f

    def inclusion_tag(self, template):
        """Adds a function to Jinja, but like Django's @inclusion_tag."""
        def decorator(f):
            @functools.wraps(f)
            def wrapper(*args, **kw):
                context = f(*args, **kw)
                t = get_env().get_template(template).render(context)
                return jinja2.Markup(t)
            return self.function(wrapper)
        return decorator

register = Register()


class Loader(BaseLoader):
    is_usable = True

    def __init__(self):
        if has_engine:
            super(Loader, self).__init__(Engine.get_default())
        else:
            super(Loader, self).__init__()
        include_pattern = getattr(settings, 'JINGO_INCLUDE_PATTERN', None)
        if include_pattern:
            self.include_re = re.compile(include_pattern)
        else:
            self.include_re = None

    def _valid_template(self, template_name):
        if self.include_re:
            if not self.include_re.search(template_name):
                return False

        if hasattr(template_name, 'split'):
            app = template_name.split('/')[0]
            if app in getattr(settings, 'JINGO_EXCLUDE_APPS', EXCLUDE_APPS):
                return False

        return True

    def load_template(self, template_name, template_dirs=None):
        if not self._valid_template(template_name):
            raise TemplateDoesNotExist(template_name)

        try:
            template = get_env().get_template(template_name)
            return template, template.filename
        except jinja2.TemplateNotFound:
            raise TemplateDoesNotExist(template_name)

    def load_template_source(self, template_name, template_dirs=None):
        if not self._valid_template(template_name):
            raise TemplateDoesNotExist(template_name)

        try:
            template = get_env().get_template(template_name)
        except jinja2.TemplateNotFound:
            raise TemplateDoesNotExist(template_name)

        with open(template.filename, 'rb') as fp:
            return (fp.read().decode(settings.FILE_CHARSET), template.filename)



import os

import nose
import django

NAME = os.path.basename(os.path.dirname(__file__))
ROOT = os.path.abspath(os.path.dirname(__file__))

os.environ['DJANGO_SETTINGS_MODULE'] = 'fake_settings'
os.environ['PYTHONPATH'] = os.pathsep.join([ROOT,
                                            os.path.join(ROOT, 'examples')])

if __name__ == '__main__':
    if hasattr(django, 'setup'):
        # Django's app registry was added in 1.7. We need to call `setup` to
        # initiate it.
        django.setup()
    nose.main()



from __future__ import unicode_literals

from django.utils import translation

try:
    from unittest.mock import sentinel
except ImportError:
    from mock import sentinel
from nose.tools import eq_

from jingo import get_env, render_to_string


def test_template_substitution_crash():
    translation.activate('xx')

    env = get_env()

    # The localized string has the wrong variable name in it
    s = '{% trans string="heart" %}Broken {{ string }}{% endtrans %}'
    template = env.from_string(s)
    rendered = render_to_string(sentinel.request, template, {})
    eq_(rendered, 'Broken heart')



from django.test.html import HTMLParseError, parse_html
from nose.tools import eq_

from jingo import get_env


def htmleq_(html1, html2, msg=None):
    """
    Asserts that two HTML snippets are semantically the same.
    Whitespace in most cases is ignored, and attribute ordering is not
    significant. The passed-in arguments must be valid HTML.

    See ticket 16921: https://code.djangoproject.com/ticket/16921

    """
    dom1 = assert_and_parse_html(html1, msg,
                                 'First argument is not valid HTML:')
    dom2 = assert_and_parse_html(html2, msg,
                                 'Second argument is not valid HTML:')

    eq_(dom1, dom2)


def assert_and_parse_html(html, user_msg, msg):
    try:
        dom = parse_html(html)
    except HTMLParseError as e:
        standard_msg = '%s\n%s\n%s' % (user_msg, msg, e.msg)
        raise AssertionError(standard_msg)
    return dom


def render(s, context={}):
    t = get_env().from_string(s)
    return t.render(context)



from __future__ import unicode_literals

from django.conf.urls import patterns


urlpatterns = patterns('',
    (r'^url/(\d+)/(\w+)/$', lambda r: None, {}, "url-args"),
    (r'^url/(?P<num>\d+)/(?P<word>\w+)/$', lambda r: None, {}, "url-kwargs"),
)



from __future__ import unicode_literals

from django.shortcuts import render

from nose.tools import eq_
try:
    from unittest.mock import Mock
except ImportError:
    from mock import Mock


def test_render():
    r = render(Mock(), 'jinja_app/test.html', {})
    eq_(r.content, b'HELLO')


def test_render_no_toplevel_override():
    r = render(Mock(), 'jinja_app/test_nonoverride.html', {})
    eq_(r.content, b'HELLO')


def test_render_toplevel_override():
    r = render(Mock(), 'jinja_app/test_override.html', {})
    eq_(r.content, b'HELLO')


def test_render_django():
    r = render(Mock(), 'django_app/test.html', {})
    eq_(r.content, b'HELLO ...\n')


def test_render_django_no_toplevel_override():
    r = render(Mock(), 'django_app/test_nonoverride.html', {})
    eq_(r.content, b'HELLO ...\n')


def test_render_django_toplevel_override():
    r = render(Mock(), 'django_app/test_override.html', {})
    eq_(r.content, b'HELLO ...\n')



"""
Django marks its form HTML "safe" according to its own rules, which Jinja2 does
not recognize.

This monkeypatches Django to support the ``__html__`` protocol used in Jinja2
templates. ``Form``, ``BoundField``, ``ErrorList``, and other form objects that
render HTML through their ``__unicode__`` method are extended with ``__html__``
so they can be rendered in Jinja2 templates without adding ``|safe``.

Call the ``patch()`` function to execute the patch. It must be called
before ``django.forms`` is imported for the conditional_escape patch to work
properly. The root URLconf is the recommended location for calling ``patch()``.

Usage::

    import jingo.monkey
    jingo.monkey.patch()

This patch was originally developed by Jeff Balogh and this version is taken
from the nuggets project at
https://github.com/mozilla/nuggets/blob/master/safe_django_forms.py

"""

from __future__ import unicode_literals

import django.utils.encoding
import django.utils.html
import django.utils.safestring
from django.utils import six

try:
    # This was removed in Django 1.5+. We add it back to support Django 1.4.
    from django.utils.encoding import StrAndUnicode
    has_str_and_unicode = True
except ImportError:
    has_str_and_unicode = False
    from django.utils.encoding import python_2_unicode_compatible

    @python_2_unicode_compatible
    class StrAndUnicode(object):
        def __str__(self):
            return self.code


# This function gets directly imported within Django, so this change needs to
# happen before too many Django imports happen.
def conditional_escape(html):
    """
    Similar to escape(), except that it doesn't operate on pre-escaped strings.
    """
    if hasattr(html, '__html__'):
        return html.__html__()
    elif isinstance(html, django.utils.safestring.SafeData):
        return html
    return django.utils.html.escape(html)


# Django uses SafeData to mark a string that has already been escaped or
# otherwise deemed safe. This __html__ method lets Jinja know about that too.
def __html__(self):
    """
    Returns the html representation of a string.

    Allows interoperability with other template engines.
    """
    return six.text_type(self)


# Django uses StrAndUnicode for classes like Form, BoundField, Widget which
# have a __unicode__ method which returns escaped html. We replace
# StrAndUnicode with SafeStrAndUnicode to get the __html__ method.
class SafeStrAndUnicode(StrAndUnicode):
    """A class whose __str__ and __html__ returns __unicode__."""

    def __html__(self):
        return six.text_type(self)


def patch():
    django.utils.html.conditional_escape = conditional_escape
    django.utils.safestring.SafeData.__html__ = __html__

    # forms imports have to come after we patch conditional_escape.
    from django.forms import forms, formsets, util, widgets

    # Replace StrAndUnicode with SafeStrAndUnicode in the inheritance
    # for all these classes.
    classes = [
        forms.BaseForm,
        forms.BoundField,
        formsets.BaseFormSet,
        util.ErrorDict,
        util.ErrorList,
        widgets.Media,
        widgets.RadioFieldRenderer,
    ]
    try:
        classes.append(widgets.RadioChoiceInput)
    except AttributeError:
        classes.append(widgets.RadioInput)

    if has_str_and_unicode:
        for cls in classes:
            bases = list(cls.__bases__)
            if StrAndUnicode in bases:
                idx = bases.index(StrAndUnicode)
                bases[idx] = SafeStrAndUnicode
                cls.__bases__ = tuple(bases)
    for cls in classes:
        if not hasattr(cls, '__html__'):
            cls.__html__ = __html__



# -*- coding: utf-8 -*-
"""Tests for the jingo's builtin helpers."""

from __future__ import unicode_literals

import cgi
from datetime import datetime
from collections import namedtuple

from django.http import QueryDict
from django.utils import six
from jinja2 import Markup
try:
    from unittest.mock import patch
except ImportError:
    from mock import patch
from nose.tools import eq_

from jingo import helpers
from jingo import register

from .utils import htmleq_, render


def test_f():
    s = render('{{ "{0} : {z}"|f("a", z="b") }}')
    eq_(s, 'a : b')


def test_f_unicode():
    s = render('{{ "foo {0}"|f(bar) }}', {'bar': u'bar\xe9'})
    eq_(s, u'foo bar\xe9')
    s = render('{{ t|f(bar) }}', {'t': u'\xe9 {0}', 'bar': 'baz'})
    eq_(s, u'\xe9 baz')


def test_f_markup():
    format_string = 'Hello <b>{0}</b>'
    format_markup = Markup(format_string)
    val_string = '<em>Steve</em>'
    val_markup = Markup(val_string)
    template = '{{ fmt|f(val) }}'
    expect = 'Hello &lt;b&gt;&lt;em&gt;Steve&lt;/em&gt;&lt;/b&gt;'

    combinations = (
        (format_string, val_string),
        (format_string, val_markup),
        (format_markup, val_string),
        (format_markup, val_markup),
    )

    def _check(f, v):
        s = render(template, {'fmt': f, 'val': v})
        eq_(expect, s)

    for f, v in combinations:
        yield _check, f, v


def test_fe_helper():
    context = {'var': '<bad>'}
    template = '{{ "<em>{t}</em>"|fe(t=var) }}'
    eq_('<em>&lt;bad&gt;</em>', render(template, context))


def test_fe_positional():
    context = {'var': '<bad>'}
    template = '{{ "<em>{0}</em>"|fe(var) }}'
    eq_('<em>&lt;bad&gt;</em>', render(template, context))


def test_fe_unicode():
    context = {'var': 'Français'}
    template = '{{ "Speak {0}"|fe(var) }}'
    eq_('Speak Français', render(template, context))


def test_fe_markup():
    format_string = 'Hello <b>{0}</b>'
    format_markup = Markup(format_string)
    val_string = '<em>Steve</em>'
    val_markup = Markup(val_string)
    template = '{{ fmt|fe(val) }}'
    expect_esc = 'Hello <b>&lt;em&gt;Steve&lt;/em&gt;</b>'
    expect_noesc = 'Hello <b><em>Steve</em></b>'

    combinations = (
        (format_string, val_string, expect_esc),
        (format_string, val_markup, expect_noesc),
        (format_markup, val_string, expect_esc),
        (format_markup, val_markup, expect_noesc),
    )

    def _check(f, v, e):
        s = render(template, {'fmt': f, 'val': v})
        eq_(e, s)

    for f, v, e in combinations:
        yield _check, f, v, e


def test_nl2br():
    text = "some\ntext\n\nwith\nnewlines"
    s = render('{{ x|nl2br }}', {'x': text})
    eq_(s, "some<br>text<br><br>with<br>newlines")

    text = None
    s = render('{{ x|nl2br }}', {'x': text})
    eq_(s, '')


def test_datetime():
    time = datetime(2009, 12, 25, 10, 11, 12)
    s = render('{{ d|datetime }}', {'d': time})
    eq_(s, 'December 25, 2009')

    s = render('{{ d|datetime("%Y-%m-%d %H:%M:%S") }}', {'d': time})
    eq_(s, '2009-12-25 10:11:12')

    s = render('{{ None|datetime }}')
    eq_(s, '')


def test_datetime_unicode():
    fmt = u"%Y 年 %m 月 %e 日"
    helpers.datetime(datetime.now(), fmt)


def test_ifeq():
    eq_context = {'a': 1, 'b': 1}
    neq_context = {'a': 1, 'b': 2}

    s = render('{{ a|ifeq(b, "<b>something</b>") }}', eq_context)
    eq_(s, '<b>something</b>')

    s = render('{{ a|ifeq(b, "<b>something</b>") }}', neq_context)
    eq_(s, '')


def test_class_selected():
    eq_context = {'a': 1, 'b': 1}
    neq_context = {'a': 1, 'b': 2}

    s = render('{{ a|class_selected(b) }}', eq_context)
    eq_(s, 'class="selected"')

    s = render('{{ a|class_selected(b) }}', neq_context)
    eq_(s, '')


def test_csrf():
    s = render('{{ csrf() }}', {'csrf_token': 'fffuuu'})
    csrf = "<input type='hidden' name='csrfmiddlewaretoken' value='fffuuu' />"
    assert csrf in s


class field(object):
    def __init__(self):
        self.field = namedtuple('_', 'widget')
        self.field.widget = namedtuple('_', 'attrs')
        self.field.widget.attrs = {'class': 'foo'}

    def __str__(self):
        attrs = self.field.widget.attrs
        attr_str = ' '.join('%s="%s"' % (k, v)
                            for (k, v) in six.iteritems(attrs))
        return Markup('<input %s />' % attr_str)

    def __html__(self):
        return self.__str__()


def test_field_attrs():
    f = field()
    s = render('{{ field|field_attrs(class="bar",name="baz") }}',
               {'field': f})
    htmleq_(s, '<input class="bar" name="baz" />')


def test_field_attrs_none():
    f = field()
    s = render('{{ field|field_attrs(class=None,x=None) }}', {'field': f})
    htmleq_(s, '<input  />')


def test_url():
    # urls defined in jingo/tests/urls.py
    s = render('{{ url("url-args", 1, "foo") }}')
    eq_(s, "/url/1/foo/")
    s = render('{{ url("url-kwargs", word="bar", num=1) }}')
    eq_(s, "/url/1/bar/")


def url(x, *y, **z):
    return '/' + x + '!'


@patch('django.conf.settings')
def test_custom_url(s):
    # register our url method
    register.function(url)
    # re-register Jinja's
    register.function(helpers.url, override=False)

    # urls defined in jingo/tests/urls.py
    s = render('{{ url("url-args", 1, "foo") }}')
    eq_(s, "/url-args!")
    s = render('{{ url("url-kwargs", word="bar", num=1) }}')
    eq_(s, "/url-kwargs!")

    # teardown
    register.function(helpers.url, override=True)


def test_filter_override():
    def f(s):
        return s.upper()
    # See issue 7688: http://bugs.python.org/issue7688
    f.__name__ = 'a' if six.PY3 else b'a'
    register.filter(f)
    s = render('{{ s|a }}', {'s': 'Str'})
    eq_(s, 'STR')

    def g(s):
        return s.lower()
    g.__name__ = 'a' if six.PY3 else b'a'
    register.filter(override=False)(g)
    s = render('{{ s|a }}', {'s': 'Str'})
    eq_(s, 'STR')

    register.filter(g)
    s = render('{{ s|a }}', {'s': 'Str'})
    eq_(s, 'str')


def _check_query(path, result=None, **query):
    paramed = helpers.urlparams(path, **query)
    if result is None:
        result = query
    for k, v in result.items():
        if not isinstance(v, list):
            result[k] = [v]
    qs = cgi.parse_qs(paramed.split('?')[1])
    for k in query:
        eq_(set(result[k]), set(qs[k]))


def test_urlparams_unicode():
    context = {'q': u'Fran\xe7ais'}
    eq_(u'/foo?q=Fran%C3%A7ais', helpers.urlparams('/foo', **context))
    context['q'] = u'\u0125help'
    eq_(u'/foo?q=%C4%A5help', helpers.urlparams('/foo', **context))


def test_urlparams_valid():
    _check_query('/foo', a='foo', b='bar')


def test_urlparams_query_string():
    _check_query('/foo?a=foo', result={'a': 'foo', 'b': 'bar'}, b='bar')


def test_urlparams_multivalue():
    _check_query('/foo?a=foo&a=bar', result={'a': ['foo', 'bar']})
    _check_query('/foo', a=['foo', 'bar'])
    eq_(u'/foo?a=bar', helpers.urlparams('/foo?a=foo', a='bar'))


def test_urlparams_none():
    """A value of None doesn't make it into the query string."""
    eq_(u'/foo', helpers.urlparams('/foo', bar=None))
    eq_(u'/foo', helpers.urlparams('/foo?a=bar', a=None))


def test_urlparams_fragment():
    eq_(u'/#foo', helpers.urlparams('/', fragment='foo'))
    eq_(u'/#bar', helpers.urlparams('/#foo', fragment='bar'))
    eq_(u'/', helpers.urlparams('/#foo', fragment=''))



from __future__ import unicode_literals

from django import forms
from django.utils import six

from jinja2 import escape
from nose.tools import eq_

import jingo.monkey

from .utils import render


class MyForm(forms.Form):
    email = forms.EmailField()


def test_monkey_patch():
    form = MyForm()
    html = form.as_ul()
    context = {'form': form}
    t = '{{ form.as_ul() }}'

    eq_(escape(html), render(t, context))

    jingo.monkey.patch()
    eq_(html, render(t, context))

    s = six.text_type(form['email'])
    eq_(s, render('{{ form.email }}', {'form': form}))



from setuptools import setup


setup(
    name='jingo',
    version='0.8',
    description='An adapter for using Jinja2 templates with Django.',
    long_description=open('README.rst').read(),
    author='Jeff Balogh',
    author_email='jbalogh@mozilla.com',
    url='http://github.com/jbalogh/jingo',
    license='BSD',
    packages=['jingo'],
    include_package_data=True,
    zip_safe=False,
    install_requires=['jinja2'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        # I don't know what exactly this means, but why not?
        'Environment :: Web Environment :: Mozilla',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



# coding: utf-8

from __future__ import unicode_literals, print_function

try:
    import urlparse
except ImportError:
    import urllib.parse as urlparse

from django.core.urlresolvers import reverse
from django.http import QueryDict
from django.template.defaulttags import CsrfTokenNode
from django.utils import six
from django.utils.encoding import smart_str
try:
    from django.utils.encoding import smart_unicode as smart_text
except ImportError:
    from django.utils.encoding import smart_text
from django.utils.http import urlencode
from django.utils.translation import ugettext as _

import jinja2

from jingo import register


@register.function
@jinja2.contextfunction
def csrf(context):
    """Equivalent of Django's ``{% crsf_token %}``."""
    return jinja2.Markup(CsrfTokenNode().render(context))


@register.filter
def f(s, *args, **kwargs):
    """
    Uses ``str.format`` for string interpolation.

    **Note**: Always converts to s to text type before interpolation.

    >>> {{ "{0} arguments and {x} arguments"|f('positional', x='keyword') }}
    "positional arguments and keyword arguments"
    """
    s = six.text_type(s)
    return s.format(*args, **kwargs)


@register.filter
def fe(s, *args, **kwargs):
    """Format a safe string with potentially unsafe arguments, then return a
    safe string."""

    s = six.text_type(s)

    args = [jinja2.escape(smart_text(v)) for v in args]

    for k in kwargs:
        kwargs[k] = jinja2.escape(smart_text(kwargs[k]))

    return jinja2.Markup(s.format(*args, **kwargs))


@register.filter
def nl2br(string):
    """Turn newlines into <br>."""
    if not string:
        return ''
    return jinja2.Markup('<br>'.join(jinja2.escape(string).splitlines()))


@register.filter
def datetime(t, fmt=None):
    """Call ``datetime.strftime`` with the given format string."""
    if fmt is None:
        fmt = _(u'%B %e, %Y')
    if not six.PY3:
        # The datetime.strftime function strictly does not
        # support Unicode in Python 2 but is Unicode only in 3.x.
        fmt = fmt.encode('utf-8')
    return smart_text(t.strftime(fmt)) if t else ''


@register.filter
def ifeq(a, b, text):
    """Return ``text`` if ``a == b``."""
    return jinja2.Markup(text if a == b else '')


@register.filter
def class_selected(a, b):
    """Return ``'class="selected"'`` if ``a == b``."""
    return ifeq(a, b, 'class="selected"')


@register.filter
def field_attrs(field_inst, **kwargs):
    """Adds html attributes to django form fields"""
    for k, v in kwargs.items():
        if v is not None:
            field_inst.field.widget.attrs[k] = v
        else:
            try:
                del field_inst.field.widget.attrs[k]
            except KeyError:
                pass
    return field_inst


@register.function(override=False)
def url(viewname, *args, **kwargs):
    """Return URL using django's ``reverse()`` function."""
    return reverse(viewname, args=args, kwargs=kwargs)


@register.filter
def urlparams(url_, fragment=None, query_dict=None, **query):
    """
Add a fragment and/or query parameters to a URL.

New query params will be appended to exising parameters, except duplicate
names, which will be replaced.
"""
    url_ = urlparse.urlparse(url_)
    fragment = fragment if fragment is not None else url_.fragment

    q = url_.query
    new_query_dict = (QueryDict(smart_str(q), mutable=True) if
                      q else QueryDict('', mutable=True))
    if query_dict:
        for k, l in query_dict.lists():
            new_query_dict[k] = None  # Replace, don't append.
            for v in l:
                new_query_dict.appendlist(k, v)

    for k, v in query.items():
        # Replace, don't append.
        if isinstance(v, list):
            new_query_dict.setlist(k, v)
        else:
            new_query_dict[k] = v

    query_string = urlencode([(k, v) for k, l in new_query_dict.lists() for
                              v in l if v is not None])
    new = urlparse.ParseResult(url_.scheme, url_.netloc, url_.path,
                               url_.params, query_string, fragment)
    return new.geturl()



import os

path = lambda *a: os.path.join(ROOT, *a)

ROOT = os.path.dirname(os.path.abspath(__file__))
INSTALLED_APPS = (
    'django.contrib.admin.apps.SimpleAdminConfig',
    'jingo.tests.jinja_app',
    'jingo.tests.django_app',
)
TEMPLATE_LOADERS = (
    'jingo.Loader',
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)
TEMPLATE_DIRS = (path('jingo/tests/templates'),)
JINGO_EXCLUDE_APPS = ('django_app',)
ROOT_URLCONF = 'jingo.tests.urls'

SECRET_KEY = 'jingo'



from __future__ import unicode_literals

from django.shortcuts import render
import jinja2

from nose.tools import eq_
try:
    from unittest.mock import Mock, patch, sentinel
except ImportError:
    from mock import Mock, patch, sentinel

import jingo


@patch('jingo.get_env')
def test_render(mock_get_env):
    mock_template = Mock()
    mock_env = Mock()
    mock_env.get_template.return_value = mock_template
    mock_get_env.return_value = mock_env

    response = render(Mock(), sentinel.template, status=32)
    mock_env.get_template.assert_called_with(sentinel.template)
    assert mock_template.render.called

    eq_(response.status_code, 32)


@patch('jingo.get_env')
def test_render_to_string(mock_get_env):
    template = jinja2.environment.Template('The answer is {{ answer }}')
    rendered = jingo.render_to_string(Mock(), template, {'answer': 42})

    eq_(rendered, 'The answer is 42')


def test_inclusion_tag():
    @jingo.register.inclusion_tag('xx.html')
    def tag(x):
        return {'z': x}

    env = jingo.get_env()
    with patch.object(env, 'get_template') as mock_get_template:
        temp = jinja2.environment.Template('<{{ z }}>')
        mock_get_template.return_value = temp
        t = env.from_string('{{ tag(1) }}')
        eq_('<1>', t.render())
